@@ -24,7 +24,10 @@ import six
 import testtools
 
 from neutron.agent.l3 import agent as neutron_l3_agent
+from neutron.agent.l3 import dvr_edge_ha_router as dvr_ha_router
+from neutron.agent.l3 import dvr_edge_router
 from neutron.agent.l3 import dvr_fip_ns
+from neutron.agent.l3 import dvr_local_router
 from neutron.agent.l3 import dvr_snat_ns
 from neutron.agent.l3 import namespaces
 from neutron.agent.linux import ip_lib
@@ -1124,11 +1127,11 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self.assertFalse(sg_device)
         self.assertTrue(qg_device)
 
-    def _mocked_dvr_ha_router(self, agent, enable_gw=True,
+    def _mocked_dvr_ha_router(self, agent, enable_ha=True, enable_gw=True,
                               enable_centralized_fip=False,
                               snat_bound_fip=False):
         r_info = self.generate_dvr_router_info(
-            enable_ha=True,
+            enable_ha=enable_ha,
             enable_snat=True,
             agent=agent,
             enable_gw=enable_gw,
@@ -1164,7 +1167,7 @@ class TestDvrRouter(framework.L3AgentTestFramework):
 
     def _create_dvr_ha_router(self, agent, enable_gw=True,
                               enable_centralized_fip=False,
-                              snat_bound_fip=False):
+                              snat_bound_fip=False, ha_interface=True):
         get_ns_name = mock.patch.object(namespaces.RouterNamespace,
                                         '_get_ns_name').start()
         get_snat_ns_name = mock.patch.object(dvr_snat_ns.SnatNamespace,
@@ -1172,7 +1175,11 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         (r_info,
          mocked_r_ns_name,
          mocked_r_snat_ns_name) = self._mocked_dvr_ha_router(
-             agent, enable_gw, enable_centralized_fip, snat_bound_fip)
+             agent, ha_interface, enable_gw, enable_centralized_fip,
+             snat_bound_fip)
+
+        if not ha_interface:
+            r_info['ha'] = True
 
         get_ns_name.return_value = mocked_r_ns_name
         get_snat_ns_name.return_value = mocked_r_snat_ns_name
@@ -1272,6 +1279,40 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self._assert_no_ip_addresses_on_interface(namespace,
                                                   ex_gw_port_name)
 
+    @mock.patch.object(dvr_local_router.DvrLocalRouter, 'connect_rtr_2_fip')
+    @mock.patch.object(
+        dvr_ha_router.DvrEdgeHaRouter, '_get_centralized_fip_cidr_set')
+    def test_dvr_ha_router_with_centralized_fip_calls_keepalived_cidr(
+        self, connect_rtr_2_fip_mock, fip_cidr_centralized_mock):
+
+        self._setup_dvr_ha_agents()
+        self._setup_dvr_ha_bridges()
+
+        router1 = self._create_dvr_ha_router(
+            self.agent, enable_gw=True,
+            enable_centralized_fip=True,
+            snat_bound_fip=True)
+        self.assertTrue(fip_cidr_centralized_mock.called)
+        restarted_agent = neutron_l3_agent.L3NATAgentWithStateReport(
+            self.agent.host, self.agent.conf)
+        self.manage_router(restarted_agent, router1.router)
+        self.assertTrue(fip_cidr_centralized_mock.called)
+
+    @mock.patch.object(dvr_local_router.DvrLocalRouter, 'connect_rtr_2_fip')
+    @mock.patch.object(
+        dvr_edge_router.DvrEdgeRouter, '_get_centralized_fip_cidr_set')
+    def test_dvr_router_with_centralized_fip_calls_keepalived_cidr(
+        self, connect_rtr_2_fip_mock, fip_cidr_centralized_mock):
+
+        router_info = self.generate_dvr_router_info(
+            enable_gw=True, enable_centralized_fip=True, snat_bound_fip=True)
+        router1 = self.manage_router(self.agent, router_info)
+        self.assertTrue(fip_cidr_centralized_mock.called)
+        restarted_agent = neutron_l3_agent.L3NATAgentWithStateReport(
+            self.agent.host, self.agent.conf)
+        self.manage_router(restarted_agent, router1.router)
+        self.assertTrue(fip_cidr_centralized_mock.called)
+
     def _test_dvr_ha_router_failover_with_gw_and_fip(self, enable_gw,
                                                      enable_centralized_fip,
                                                      snat_bound_fip):
@@ -1330,6 +1371,26 @@ class TestDvrRouter(framework.L3AgentTestFramework):
     def test_dvr_ha_router_failover_without_gw(self):
         self._test_dvr_ha_router_failover(enable_gw=False)
 
+    def test_dvr_non_ha_router_update(self):
+        self._setup_dvr_ha_agents()
+        self._setup_dvr_ha_bridges()
+
+        router1 = self._create_dvr_ha_router(self.agent)
+        router2 = self._create_dvr_ha_router(self.failover_agent,
+                                             ha_interface=False)
+
+        r1_chsfr = mock.patch.object(self.agent,
+                                     'check_ha_state_for_router').start()
+        r2_chsfr = mock.patch.object(self.failover_agent,
+                                     'check_ha_state_for_router').start()
+
+        utils.wait_until_true(lambda: router1.ha_state == 'master')
+
+        self.agent._process_updated_router(router1.router)
+        self.assertTrue(r1_chsfr.called)
+        self.failover_agent._process_updated_router(router2.router)
+        self.assertFalse(r2_chsfr.called)
+
     def _setup_dvr_router_static_routes(
         self, router_namespace=True, check_fpr_int_rule_delete=False):
         """Test to validate the extra routes on dvr routers."""
@@ -1522,9 +1583,6 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         self.agent.conf.agent_mode = 'dvr'
         router_info = self.generate_dvr_router_info(
             enable_snat=True, enable_gw=True, enable_floating_ip=True)
-        fip_agent_gw_port = router_info[n_const.FLOATINGIP_AGENT_INTF_KEY]
-        self.mock_plugin_api.get_agent_gateway_port.return_value = (
-            fip_agent_gw_port[0])
         router_info[lib_constants.FLOATINGIP_KEY] = []
         if address_scopes:
             address_scope1 = {
@@ -1542,6 +1600,12 @@ class TestDvrRouter(framework.L3AgentTestFramework):
             address_scope1)
         router_info[lib_constants.INTERFACE_KEY][1]['address_scopes'] = (
             address_scope2)
+        # should have the same address_scopes as gw_port
+        fip_agent_gw_ports = router_info[n_const.FLOATINGIP_AGENT_INTF_KEY]
+        fip_agent_gw_ports[0]['address_scopes'] = (
+            router_info['gw_port']['address_scopes'])
+        self.mock_plugin_api.get_agent_gateway_port.return_value = (
+            fip_agent_gw_ports[0])
         router1 = self.manage_router(self.agent, router_info)
         fip_ns_name = router1.fip_ns.get_name()
         self.assertTrue(self._namespace_exists(router1.ns_name))
@@ -1576,7 +1640,7 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         # Now remove the gateway and validate if the respective interface
         # routes in router namespace is deleted respectively.
         self. _assert_interface_rules_on_gateway_remove(
-            router1, self.agent, address_scopes, fip_agent_gw_port[0],
+            router1, self.agent, address_scopes, fip_agent_gw_ports[0],
             rfp_device, fpr_device)
 
     def test_dvr_fip_and_router_namespace_rules_with_address_scopes_match(
@@ -1644,6 +1708,10 @@ class TestDvrRouter(framework.L3AgentTestFramework):
         if gw_address_scope:
             router_info['gw_port']['address_scopes'] = {
                 str(lib_constants.IP_VERSION_4): gw_address_scope}
+            fip_agent_gw_ports = router_info[
+                n_const.FLOATINGIP_AGENT_INTF_KEY]
+            fip_agent_gw_ports[0]['address_scopes'] = router_info['gw_port'][
+                'address_scopes']
         router_info[lib_constants.INTERFACE_KEY][0]['address_scopes'] = (
             address_scope1)
         router_info[lib_constants.INTERFACE_KEY][1]['address_scopes'] = (
